use axum::extract::State;
use axum::response::Redirect;
use axum::routing::post;
use axum::Form;
use axum::{routing::get, Router};
use chrono::prelude::*;
use chrono::serde::ts_seconds_option;
use maud::{html, Markup, DOCTYPE};
use serde::{Deserialize, Serialize};
use sqlx::{query, query_as, SqlitePool};
use tower_http::services::ServeDir;
use tracing::{debug, info, instrument, trace};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

#[tokio::main]
async fn main() {
    tracing_subscriber::registry()
        .with(
            tracing_subscriber::EnvFilter::try_from_default_env()
                .unwrap_or_else(|_| "debug".into()),
        )
        .with(tracing_subscriber::fmt::layer())
        .init();

    let dbpool = SqlitePool::connect("fam.db").await.unwrap();
    let _ = sqlx::migrate!().run(&dbpool).await;

    debug!("Debug showing");
    trace!("Trace showing");

    // build our application with a route
    let app = Router::new()
        .route("/", get(hello_world))
        .route("/admin", get(admin))
        .route("/admin/new-list", post(new_list))
        .with_state(dbpool)
        .nest_service("/assets", ServeDir::new("assets"));

    let bind = "0.0.0.0:3000";
    let listener = tokio::net::TcpListener::bind(bind).await.unwrap();
    info!("Listening on {bind}");
    axum::serve(listener, app).await.unwrap();
}

#[derive(sqlx::FromRow, Debug, Serialize, Deserialize)]
struct FamilyMember {
    avatar_url: String,
    name: String,
}

#[derive(sqlx::FromRow, Debug, Serialize, Deserialize)]
struct Todo {
    emoji: char,
    text: String,
    #[serde(with = "ts_seconds_option")]
    done_at: Option<DateTime<Utc>>,
}

fn head() -> Markup {
    html! {
        (DOCTYPE)
        head {
            meta charset="utf-8" {}
            meta name="viewport" content="width=device-width, initial-scale=1" {}
            link rel="stylesheet" href="/assets/style.css" {}
            script src="/assets/htmx-1.9.10.js" {}
            title { "Homeman" }
        }
    }
}

#[instrument]
async fn hello_world() -> Markup {
    trace!("Hello world page accessed");
    html! {
        (head())
        body {
            h1 class="title" { "Flanagan Family" }
            h1 class="title" { "1:05pm" }
            h1 class="title" { "To-Do" }
        }
    }
}

async fn admin(State(dbpool): State<SqlitePool>) -> Markup {
    trace!("Admin page accessed");
    let family_members: Vec<FamilyMember> = query_as!(
        FamilyMember,
        "select name, avatar_url from 'family_members';"
    )
    .fetch_all(&dbpool)
    .await
    .unwrap();
    html! {
        (head())
        body {
            h1 class="title" { "Flanagan Family ADMIN" }
            h1 class="title" { "Members" }
            ul {
                @for fam in &family_members {
                    li { (fam.name) }
                }
            }
            form action="/admin/new-list" method="POST" {
                label {
                    "Name"
                    input id="name" name="name" type="text" placeholder="John Smith" {}
                    input id="avatar_url" name="avatar_url" type="text" placeholder="/assets/yo.png" {}
                }
                button type="submit" {"Add Member"}
            }
        }
    }
}

async fn new_list(State(dbpool): State<SqlitePool>, Form(fm): Form<FamilyMember>) -> Redirect {
    trace!("New list creation attempt");
    let _ = query!(
        "insert into family_members (name, avatar_url) values (?1, ?2);",
        fm.name,
        fm.avatar_url
    )
    .execute(&dbpool)
    .await
    .unwrap();
    Redirect::to("/admin")
}
